{ LLSTAT.PAS : Line Length statistics of a text file

  Title   : LLSTAT
  Language: Borland Pascal v5.0 or higher
  Version : 1.1
  Date    : Feb 15, 2000
  Author  : J R Ferguson
  Download: http://hello.to/ferguson
  E-mail  : j.r.ferguson@iname.com
  Usage   : LLSTAT infile [/S]
  Options : /S : skip empty lines


This program and its source may be used and copied freely without charge,
but  only  for non-commercial purposes. The author is not responsible for
any damage or loss of data that may be caused by using it.

To compile this source file, you wil need  some  units  from  the  JRFPAS
Pascal  routine  library by the same author, which can be downloaded from
the Internet address mentioned above.
}

{$V-}
{$R+}

program LLSTAT;
uses ArgLib, StpLib, ChrLib, DefLib;

const
  { Option defaults: }
  DFLSKIP   = false;

  MAXFNAME  = 79;

  { Error codes and messages: }
  ERROK     = 0;
  ERRARG    = 1;
  ERRINP    = 2;
  ERROUT    = 3;

  ERRMSG    : array[ERRINP..ERROUT] of StpTyp =
 ('File not found : ',
  'Can''t open output : '
 );


var
  InpFname,
  OutFname  : StpTyp;
  InpFvar,
  OutFvar   : Text;
  InpOpen,
  OutOpen   : boolean;
  InpIObuf  : array[1..8192] of char;
  ErrCod    : integer;
  OptSkip   : boolean;
  MinLen,
  MaxLen    : StpInd;
  TotCnt    : LongInt;
  TotLen,
  TotSqr    : Real;


procedure Help;
begin
  writeln('LLSTAT: Line length statistics of a text file');
  writeln('Usage : LLSTAT infile [/S]');
  writeln('Option: /S : skip empty lines');
end;

{--- Command line parsing ---}

procedure ReadOpt(arg: StpTyp);
begin
  StpDel(arg,1,1);
  if StpLen(arg) <> 1 then ErrCod:= ERRARG
  else case ToUpper(StpcGet(arg)) of
    'S' : OptSkip := not DFLSKIP;
    else ErrCod:= ERRARG;
  end;
end;


procedure ReadArgs;
var i   : ArgInd;
    arg : StpTyp;
begin
  StpCreate(InpFname); StpCreate(OutFname); { Standard input/output }
  GetArgs;
  i:= 0;
  while (i < ArgC) and (ErrCod = ERROK) do begin
    Inc(i); StpCpy(arg,ArgV[i]); StpUpp(arg);
    case StpcRet(arg,1) of
      '/' : ReadOpt(arg);
      else  StpNCpy(InpFname,arg,MAXFNAME);
    end;
  end;
end;


{--- I/O routines ---}

procedure OpenInp;
begin
  StpNCpy(InpFname,InpFname,MAXFNAME);
  {$I-} Assign(InpFvar,InpFname); SetTextBuf(InpFvar,InpIObuf);
        reset(InpFvar);
  {$I+} if IOresult <> 0 then ErrCod:= ERRINP else InpOpen:= true;
end;


procedure OpenOut;
begin
  StpNCpy(OutFname,OutFname,MAXFNAME);
  {$I-} Assign(OutFvar,OutFname); rewrite(OutFvar) {$I+};
  if IOresult <> 0 then ErrCod:= ERROUT else OutOpen:= true;
end;


{--- Main line ---}

procedure MainInit;
begin
  OptSkip:= DFLSKIP;
  ErrCod:= ERROK; InpOpen:= false; OutOpen:= false;
  ReadArgs;
  if ErrCod = ERROK then begin
    OpenInp;
    if ErrCod = ERROK then OpenOut;
  end;
end;


procedure MainTerm;
begin
  if InpOpen then Close(InpFvar);
  if OutOpen then Close(OutFvar);
  case ErrCod of
    ERROK : {do nothing};
    ERRARG: Help;
    ERRINP: writeln(ERRMSG[ErrCod],InpFName);
    ERROUT: writeln(ERRMSG[ErrCod],OutFName);
    else    writeln('Unknown error, ErrCod = ',ErrCod);
  end;
end;


procedure WriteStats;
var AvgLen,
    StdDev : Real;
begin
  WriteLn      (OutFvar,'Number of lines: ',TotCnt:6);
  if TotCnt > 0 then begin
    WriteLn    (OutFvar,'Minimum length : ',MinLen:6);
    WriteLn    (OutFvar,'Maximum length : ',MaxLen:6);
    if TotCnt > 1 then begin
      AvgLen:= TotLen/TotCnt;
      WriteLn  (OutFvar,'Average length : ',AvgLen:8:1);
      if TotCnt > 2 then begin
	StdDev:= sqrt(TotSqr/TotCnt-sqr(AvgLen));
	WriteLn(OutFvar,'Std. deviation : ',StdDev:8:1);
      end;
    end;
  end;
end;

procedure Process;
var Line   : StpTyp;
    n      : byte absolute Line;
begin
  TotCnt:= 0; TotLen:= 0.0; TotSqr:= 0.0; MinLen:= MAXSTP; MaxLen:= 0;
  while not eof(InpFvar) do begin
    ReadLn(InpFvar,Line);
    if not (OptSkip and (n=0)) then begin
      inc(TotCnt);
      TotLen:= TotLen + n;
      TotSqr:= TotSqr + sqr(n);
      if n < MinLen then MinLen:= n;
      if n > MaxLen then MaxLen:= n;
    end;
  end;
  WriteStats;
end;


begin { Main program }
  MainInit;
  if ErrCod = ERROK then Process;
  MainTerm;
end.
